@@ -5,6 +5,11 @@
  * it and/or use it and/or modify it under the terms of the 
  * "Artistic License" 
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #ifdef AUDIO_OPENAL
 
 #include "AudioOpenAL.h"
@@ -15,22 +20,28 @@
 	void	alcDestroyContext(void*) { alExit(); }
 #endif //macintosh
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstdarg>
+#include <cstring>
 
 #ifndef _WIN32
 	#include <unistd.h>
-	#include <errno.h>
+	#include <cerrno>
 	#include <fcntl.h>
 	#include <sys/stat.h>
 	#include <sys/types.h>
 #endif //_WIN32
 
-#include <AL/al.h>
-#include <AL/alc.h>
-#include <AL/alut.h>
+#ifdef __APPLE__
+	#include <OpenAL/al.h>
+	#include <OpenAL/alc.h>
+	#include <OpenAL/alut.h>
+#else
+	#include <AL/al.h>
+	#include <AL/alc.h>
+	#include <AL/alut.h>
+#endif
 
 #include "Config.h"
 
@@ -52,12 +63,17 @@
 	//try to use OpenAL alc[GS]etAudioChannel extensions in linux...
 	#ifdef __linux__ 
 		#define CD_VOLUME 1
-//		#include <AL/alext.h>
-		#include <AL/alexttypes.h>
+		#include <AL/alext.h>
+//		#include <AL/alexttypes.h>
 	#endif //__linux__
 #endif //USE_SDL
 #endif
 
+// Don't try to build the CD volume stuff if it isn't available
+#ifndef ALC_CHAN_CD_LOKI
+#undef CD_VOLUME
+#endif
+
 #ifdef __linux__
 #define USE_PLAYLIST 1
 #endif
@@ -124,6 +140,7 @@ AudioOpenAL::AudioOpenAL()
 			audioScale[0] = 0.5;
 			audioScale[1] = 0.3;
 			audioScale[2] = 0.3;
+			alDistanceModel(AL_NONE);
 		}
 
 		if(config->swapStereo())
@@ -153,6 +170,11 @@ AudioOpenAL::~AudioOpenAL()
 
 		checkError("AudioOpenAL::~Audio()");
 
+		alDeleteSources(NUM_EXPLO_POP-1, sourceExploPop);
+		alDeleteSources(NUM_EXPLO-1, sourceExplosion);
+		alDeleteSources(NumSoundTypes, source);
+		alDeleteBuffers(NumSoundTypes, buffer);
+
 		#ifdef _WIN32
 		if(context_id)
 			alcDestroyContext((ALCcontext*)context_id);
@@ -161,6 +183,8 @@ AudioOpenAL::~AudioOpenAL()
 			alcDestroyContext(context_id);
 		#endif
 		
+		alutExit();
+
 		fprintf(stderr, "done.\n");
 	}
 }
@@ -235,11 +259,15 @@ void AudioOpenAL::initSound()
 	printExtensions(stderr,  (const char*)alGetString( AL_EXTENSIONS ));
 	fprintf(stderr, "------------------------------------------------------------\n");
 
+	alutInitWithoutContext(0, NULL);
+
 	checkForExtensions();
 	
 	alListenerfv(AL_POSITION, pos);
 	
+#if !defined(ALUT_API_MAJOR_VERSION) || ALUT_API_MAJOR_VERSION < 1
 	alGenBuffers(NumSoundTypes, buffer);
+#endif
 	alGenSources(NumSoundTypes, source);
 	
 	loadSounds();
@@ -310,7 +338,9 @@ void AudioOpenAL::checkForExtensions()
 				
 	//-- check AttenuationScale extension
 	alAttenuationScale = (void (*)(ALfloat param))
-						alGetProcAddress((ALubyte *)"alAttenuationScale_LOKI");
+						alGetProcAddress("alAttenuationScale_LOKI");
+
+	alGetError(); // Don't care what the problem is
 	if(alAttenuationScale == NULL) 
 		fprintf(stderr, "ATTENTION!! Could not load alAttenuationScale\n");
 	else
@@ -318,9 +348,9 @@ void AudioOpenAL::checkForExtensions()
 	
 	//-- check Audio Channel extension
 	alcGetAudioChannel = (float (*)(ALuint channel))
-						alGetProcAddress((const ALubyte *)"alcGetAudioChannel_LOKI");
+						alGetProcAddress("alcGetAudioChannel_LOKI");
 	alcSetAudioChannel = (void (*)(ALuint channel, ALfloat volume))
-						alGetProcAddress((const ALubyte *)"alcSetAudioChannel_LOKI");
+						alGetProcAddress("alcSetAudioChannel_LOKI");
 #ifdef CD_VOLUME
 	if(alcGetAudioChannel)
 		origCDvolume = alcGetAudioChannel(ALC_CHAN_CD_LOKI);
@@ -328,10 +358,10 @@ void AudioOpenAL::checkForExtensions()
 	
 	//-- check MP3 extension
 	alutLoadMP3 = (ALboolean (*)(ALuint, ALvoid *, ALint))
-		alGetProcAddress((const ALubyte *)"alutLoadMP3_LOKI");
+		alGetProcAddress("alutLoadMP3_LOKI");
 	//-- check Ogg/Vorbis extension
 	alutLoadVorbis = (ALboolean (*)(ALuint, ALvoid *, ALint))
-		alGetProcAddress((const ALubyte *)"alutLoadVorbis_LOKI");
+		alGetProcAddress("alutLoadVorbis_LOKI");
 
 #endif //_WIN32
 }
@@ -468,9 +498,6 @@ void AudioOpenAL::setMusicVolume(float vol)
 void AudioOpenAL::loadSounds()
 {
 	int i;
-	ALsizei size, freq, bits;
-	ALenum format;
-	ALvoid *data;
 
 	for(i = 0; i < NumSoundTypes; i++)
 	{
@@ -480,10 +507,22 @@ void AudioOpenAL::loadSounds()
 		}
 		else
 		{
+#if defined(ALUT_API_MAJOR_VERSION) && ALUT_API_MAJOR_VERSION >= 1
+			buffer[i] = alutCreateBufferFromFile(dataLoc(fileNames[i]));
+			if( buffer[i] == AL_NONE ) checkError();
+#else
+			ALsizei size, freq;
+			ALenum format;
+			ALvoid *data;
+			ALboolean loop;
 #ifndef _WIN32
-			alutLoadWAV(dataLoc(fileNames[i]), &data, &format, &size, &bits, &freq);
+#ifdef __APPLE__
+			alutLoadWAVFile(const_cast<ALbyte*>(dataLoc(fileNames[i])), &format, &data, &size, &freq);
+#else
+			alutLoadWAVFile(const_cast<ALbyte*>(dataLoc(fileNames[i])), &format, &data, &size, &freq, &loop);
+#endif
 			alBufferData (buffer[i], format, data, size, freq);
-			free(data);
+			alutUnloadWAV(format,data,size,freq);
 #else //_WIN32
 			char nameBuffer[256];
 			sprintf(nameBuffer, "%s", dataLoc(fileNames[i]));
@@ -492,6 +531,7 @@ void AudioOpenAL::loadSounds()
 			alBufferData(buffer[i],format,data,size,freq);
 			alutUnloadWAV(format,data,size,freq);
 #endif//_WIN32
+#endif
 		}
 	}
 }
@@ -509,6 +549,11 @@ void AudioOpenAL::checkError(const char* tag)
 //	{
 //		fprintf(stderr, "ERROR!! <%s> alcGetError() = %s\n", tag, alcGetString(error) );
 //	}
+	error = alutGetError();
+	if(error != ALUT_ERROR_NO_ERROR)
+	{
+		fprintf(stderr, "ERROR!! <%s> alutGetError() = %s\n", tag, alutGetErrorString(error) );
+	}
 }
 
 /**
@@ -842,17 +887,29 @@ bool AudioOpenAL::loadWAV(const char *filename)
 	fprintf(stderr, "         PlayList support not compiled into Chromium (AudioOpenAL.cpp)\n");
 	return false;
 #else //USE_PLAYLIST
+#if defined(ALUT_API_MAJOR_VERSION) && ALUT_API_MAJOR_VERSION >= 1
+	buffer[MusicGame] = alutCreateBufferFromFile(const_cast<ALbyte*>(filename));
+	if( buffer[MusicGame] == AL_NONE ) checkError();
+	return buffer[MusicGame] != AL_NONE;
+#else
 	bool retVal;
-	ALsizei size, freq, bits;
+	ALsizei size, freq;
 	ALenum format;
 	ALvoid *data;
-	retVal = alutLoadWAV(filename, &data, &format, &size, &bits, &freq);
+	ALboolean loop;
+#ifdef __APPLE__
+	alutLoadWAVFile(const_cast<ALbyte*>(filename), &format, &data, &size, &freq);
+#else
+	alutLoadWAVFile(const_cast<ALbyte*>(filename), &format, &data, &size, &freq, &loop);
+#endif
+	retVal = (alGetError() == AL_NO_ERROR);
 	if(retVal)
 	{
 		alBufferData (buffer[MusicGame], format, data, size, freq);
-		free(data);	
+		alutUnloadWAV(format,data,size,freq);
 	}
 	return retVal;
+#endif
 #endif//USE_PLAYLIST
 }
 
